@@ -7,6 +7,7 @@ import (
 	"strconv"
 	"strings"
 	"testing"
+	"time"
 
 	"github.com/alecthomas/chroma/v2"
 	"github.com/alecthomas/chroma/v2/formatters"
@@ -323,21 +324,105 @@ func TestReadTextDone(t *testing.T) {
 	assert.NilError(t, testMe._wait())
 }
 
-func TestFilterNotInstalled(t *testing.T) {
-	t.Skip("FIXME: Test what happens if we try to use a filter that is not installed")
+// If people keep appending to the currently opened file we should display those
+// changes.
+func TestReadUpdatingFile(t *testing.T) {
+	// Make a temp file containing one line of text, ending with a newline
+	file, err := os.CreateTemp("", "moar-TestReadUpdatingFile-*.txt")
+	assert.NilError(t, err)
+	defer os.Remove(file.Name())
+
+	const firstLineString = "First line\n"
+	_, err = file.WriteString(firstLineString)
+	assert.NilError(t, err)
+
+	// Start a reader on that file
+	testMe, err := NewReaderFromFilename(file.Name(), *styles.Get("native"), formatters.TTY16m, nil)
+	assert.NilError(t, err)
+
+	// Wait for the reader to finish reading
+	assert.NilError(t, testMe._wait())
+	assert.Equal(t, len([]byte(firstLineString)), int(testMe.bytesCount))
+
+	// Verify we got the single line
+	allLines, _ := testMe.GetLines(linenumbers.LineNumber{}, 10)
+	assert.Equal(t, len(allLines.lines), 1)
+	assert.Equal(t, testMe.GetLineCount(), 1)
+	assert.Equal(t, allLines.lines[0].Plain(nil), "First line")
+
+	// Append a line to the file
+	const secondLineString = "Second line\n"
+	_, err = file.WriteString(secondLineString)
+	assert.NilError(t, err)
+
+	// Give the reader some time to react
+	for i := 0; i < 20; i++ {
+		allLines, _ = testMe.GetLines(linenumbers.LineNumber{}, 10)
+		if len(allLines.lines) == 2 {
+			break
+		}
+		time.Sleep(100 * time.Millisecond)
+	}
+
+	// Verify we got the two lines
+	allLines, _ = testMe.GetLines(linenumbers.LineNumber{}, 10)
+	assert.Equal(t, len(allLines.lines), 2, "Expected two lines after adding a second one, got %d", len(allLines.lines))
+	assert.Equal(t, testMe.GetLineCount(), 2)
+	assert.Equal(t, allLines.lines[0].Plain(nil), "First line")
+	assert.Equal(t, allLines.lines[1].Plain(nil), "Second line")
+
+	assert.Equal(t, int(testMe.bytesCount), len([]byte(firstLineString+secondLineString)))
 }
 
-func TestFilterFailure(t *testing.T) {
-	t.Skip("FIXME: Test what happens if the filter command fails because of bad command line options")
+// If people keep appending to the currently opened file we should display those
+// changes.
+//
+// This test verifies it with an initially empty file.
+func TestReadUpdatingFile_InitiallyEmpty(t *testing.T) {
+	// Make a temp file containing one line of text, ending with a newline
+	file, err := os.CreateTemp("", "moar-TestReadUpdatingFile_NoNewlineAtEOF-*.txt")
+	assert.NilError(t, err)
+	defer os.Remove(file.Name())
+
+	// Start a reader on that file
+	testMe, err := NewReaderFromFilename(file.Name(), *styles.Get("native"), formatters.TTY16m, nil)
+	assert.NilError(t, err)
+
+	// Wait for the reader to finish reading
+	assert.NilError(t, testMe._wait())
+
+	// Verify no lines
+	allLines, _ := testMe.GetLines(linenumbers.LineNumber{}, 10)
+	assert.Equal(t, len(allLines.lines), 0)
+	assert.Equal(t, testMe.GetLineCount(), 0)
+
+	// Append a line to the file
+	_, err = file.WriteString("Text\n")
+	assert.NilError(t, err)
+
+	// Give the reader some time to react
+	for i := 0; i < 20; i++ {
+		allLines, _ = testMe.GetLines(linenumbers.LineNumber{}, 10)
+		if len(allLines.lines) == 1 {
+			break
+		}
+		time.Sleep(100 * time.Millisecond)
+	}
+
+	// Verify we got the two lines
+	allLines, _ = testMe.GetLines(linenumbers.LineNumber{}, 10)
+	assert.Equal(t, len(allLines.lines), 1, "Expected one line after adding one, got %d", len(allLines.lines))
+	assert.Equal(t, testMe.GetLineCount(), 1)
+	assert.Equal(t, allLines.lines[0].Plain(nil), "Text")
 }
 
-func TestFilterPermissionDenied(t *testing.T) {
-	t.Skip("FIXME: Test what happens if the filter command fails because it can't access the requested file")
-}
+// NOTE: Consider testing file tailing with the first part not ending in a
+// linefeed. In this case, the last line should become longer when new bytes
+// show up.
 
-func TestFilterNotAFile(t *testing.T) {
-	t.Skip("FIXME: Test what happens if the filter command fails because the target is not a file")
-}
+// NOTE: Consider testing file tailing with the first part ending ending in the
+// middle of an UTF-8 character. In this case, the new bytes should complete the
+// UTF-8 character so it looks great again.
 
 // How long does it take to read a file?
 //
