@@ -7,7 +7,6 @@ import (
 	"regexp"
 	"strconv"
 	"strings"
-	"sync/atomic"
 	"unicode/utf8"
 
 	log "github.com/sirupsen/logrus"
@@ -70,6 +69,19 @@ type Screen interface {
 	Events() chan Event
 }
 
+type interruptableReader interface {
+	Read(p []byte) (n int, err error)
+
+	// Interrupt unblocks the read call, either now or eventually.
+	Interrupt()
+
+	// Close() should be called after you are done with the interruptableReader.
+	//
+	// It will not close the underlying reader, but it will prevent Interrupt()
+	// from hanging if called after a failure in the screen mainLoop().
+	Close() error
+}
+
 type UnixScreen struct {
 	widthAccessFromSizeOnly  int // Access from Size() method only
 	heightAccessFromSizeOnly int // Access from Size() method only
@@ -81,7 +93,7 @@ type UnixScreen struct {
 
 	events chan Event
 
-	shutdownRequested atomic.Bool
+	ttyInReader interruptableReader
 
 	ttyIn            *os.File
 	oldTerminalState *term.State //nolint Not used on Windows
@@ -150,6 +162,14 @@ func NewScreenWithMouseModeAndColorType(mouseMode MouseMode, terminalColorCount
 	if err != nil {
 		return nil, fmt.Errorf("problem setting up TTY: %w", err)
 	}
+	screen.ttyInReader, err = newInterruptableReader(screen.ttyIn)
+	if err != nil {
+		restoreErr := screen.restoreTtyInTtyOut()
+		if restoreErr != nil {
+			log.Warn("Problem restoring TTY state after failed interruptable reader setup: ", restoreErr)
+		}
+		return nil, fmt.Errorf("problem setting up TTY reader: %w", err)
+	}
 
 	screen.setAlternateScreenMode(true)
 
@@ -176,18 +196,8 @@ func (screen *UnixScreen) Close() {
 	// Tell the pager to exit unless it hasn't already
 	screen.events <- EventExit{}
 
-	// Tell our main loop to exit. Previously we used to close the screen.ttyIn
-	// file descriptor here, but:
-	// * That didn't interrupt the blocking read() in the main loop
-	// * It may or may not have caused shutdown issues on Windows
-	//
-	// Setting this flag doesn't interrupt the blocking read() either, but it
-	// is should at least be less likely to cause shutdown issues on Windows.
-	//
-	// Ref:
-	// * https://github.com/walles/moar/issues/217
-	// * https://github.com/walles/moar/issues/221
-	screen.shutdownRequested.Store(true)
+	// Tell our main loop to exit
+	screen.ttyInReader.Interrupt()
 
 	screen.hideCursor(false)
 	screen.enableMouseTracking(false)
@@ -362,6 +372,7 @@ func (screen *UnixScreen) ShowCursorAt(column int, row int) {
 
 func (screen *UnixScreen) mainLoop() {
 	defer func() {
+		screen.ttyInReader.Close()
 		log.Debug("Twin screen main loop done")
 	}()
 
@@ -375,7 +386,7 @@ func (screen *UnixScreen) mainLoop() {
 	maxBytesRead := 0
 	expectingTerminalBackgroundColor := true
 	for {
-		count, err := screen.ttyIn.Read(buffer)
+		count, err := screen.ttyInReader.Read(buffer)
 		if err != nil {
 			// Ref:
 			// * https://github.com/walles/moar/issues/145
@@ -386,12 +397,6 @@ func (screen *UnixScreen) mainLoop() {
 			screen.events <- EventExit{}
 			return
 		}
-		if screen.shutdownRequested.Load() {
-			log.Debug("Shutdown requested, twin giving up")
-
-			screen.events <- EventExit{}
-			return
-		}
 
 		if expectingTerminalBackgroundColor {
 			// This is the response to our background color request
